local L = LibStub("AceLocale-3.0"):GetLocale("QuestItemBar")

--CONFIG
QuestItemBar.defaults = {
    profile = {
		skin = {
			ID = "DreamLayout",
			Gloss = 0,
			Backdrop = true,
			Colors = {},
		},
		bar = {
			relativePoint = "CENTER",
			x = 0,
			y = 0,
			direction = "RIGHT",
			alpha = 1,
			alphaHover = 1,
			scale = 1,
			padding = 1,
			locked = false,
			hidden = false,
			onlyUsable = false,
			oorColor = { r = 1, g = 0, b = 0 }
		},
		buttonSize = 36,
    },
}
--END CONFIG

--Ripped from Recount. Thanks!
local function deepCopy(object)
	local lookup_table = {}
	local function _copy(object)
		if type(object) ~= "table" then
			return object
		elseif lookup_table[object] then
			return lookup_table[object]
		end
		local new_table = {}
		lookup_table[object] = new_table
		for index, value in pairs(object) do
			new_table[_copy(index)] = _copy(value)
		end
		return setmetatable(new_table, getmetatable(object))
	end
	return _copy(object)
end

--OPTIONS
QuestItemBar.consoleOptions = {
	type = "group",
	handler = QuestItemBar,
	args = {
		lock = {
			order = 1,
			type = "toggle",
			name = L["Lock bar"],
			desc = L["Lock bar and hide anchor."],
			set = "SetLock",
			get = function() return QuestItemBar.db.profile.bar.locked end,
		},
		hide = {
			order = 2,
			type = "toggle",
			name = L["Hide bar"],
			desc = L["Hide bar."],
			set = "SetHidden",
			get = function() return QuestItemBar.db.profile.bar.hide end,
		},
		onlyUsable = {
			order = 3,
			type = "toggle",
			name = L["Show only usable items"],
			desc = L["Show only usable items and hide all other quest items."],
			set = "SetUsable",
			get = function() return QuestItemBar.db.profile.bar.onlyUsable end,
		},
		
		styleheader = {
			order = 10,
			type = "header",
			name = L["Bar Style & Layout"],
		},
		alpha = {
			order = 11,
			name = L["Alpha"],
			desc = L["Configure the alpha of the bar."],
			type = "range",
			min = 0, max = 1, bigStep = 0.1,
			get = function() return QuestItemBar.db.profile.bar.alpha end,
			set = "SetAlpha",
		},
		alphaHover = {
			order = 12,
			name = L["Alpha on hover"],
			desc = L["Configure the alpha of the bar when mouse is over."],
			type = "range",
			min = .1, max = 1, bigStep = 0.1,
			get = function() return QuestItemBar.db.profile.bar.alphaHover end,
			set = "SetAlphaHover",
		},
		scale = {
			order = 13,
			name = L["Scale"],
			desc = L["Configure the scale of the bar."],
			type = "range",
			min = .1, max = 2, step = 0.05,
			get = function() return QuestItemBar.db.profile.bar.scale end,
			set = "SetScale",
		},
		padding = {
			order = 14,
			type = "range",
			name = L["Padding"],
			desc = L["Configure the padding of the buttons."],
			min = -10, max = 20, step = 1,
			get = function() return QuestItemBar.db.profile.bar.padding end,
			set = "SetPadding",
		},
		oorColor = {
			order = 15,
			type = "color",
			name = L["Out of range color"],
			desc = L["Configure the out of range color of the buttons."],
			get = function(info)
				local color = QuestItemBar.db.profile.bar[info[#info]]
				return color.r, color.g, color.b
			end,
			set = "SetColor",
		},
		direction = {
			order = 16,
			type = "select",
			name = L["Direction"],
			desc = L["Configure the direction of the buttons from the anchor."],
			values = { UP = L["Up"], DOWN = L["Down"], LEFT = L["Left"], RIGHT = L["Right"] },
			get = function() return QuestItemBar.db.profile.bar.direction end,
			set = "SetDirection",
		},
		config = {
			order = 17,
			name = L["Open config UI"],
			desc = L["Open the blizzard configuration UI."],
			type = "execute",
			guiHidden = true,
			func = "OpenConfig",
		},
	},
}
QuestItemBar.options = deepCopy(QuestItemBar.consoleOptions)
--END OPTIONS

function QuestItemBar:SetAlpha(_, value)
	self.db.profile.bar.alpha = value
	self.anchor:SetAlpha(value)
end

function QuestItemBar:SetAlphaHover(_, value)
	self.db.profile.bar.alphaHover = value
end

function QuestItemBar:SetScale(_, value)
	self.db.profile.bar.scale = value
	self.anchor:SetScale(value)
end

function QuestItemBar:SetPadding(_, value)
	self.db.profile.bar.padding = value
	self:UpdateBar()
end

function QuestItemBar:SetColor(info, r, g, b)
	local color = self.db.profile.bar[info[#info]]
	color.r, color.g, color.b = r, g, b
end

function QuestItemBar:SetDirection(_, value)
	self.db.profile.bar.direction = value
	self:UpdateBar()
end

function QuestItemBar:SetLock(_, value)
	QuestItemBar.db.profile.bar.locked = value
	self.anchor:SetMovable(not value)
	self:UpdateBar()
end

function QuestItemBar:ToggleLock()
	self:SetLock(_, not self.db.profile.bar.locked)
end

function QuestItemBar:SetHidden(_, value)
	QuestItemBar.db.profile.bar.hide = value
	if QuestItemBar.db.profile.bar.hide then
		QuestItemBar.anchor:Hide()
	else
		QuestItemBar.anchor:Show()
	end
end

function QuestItemBar:ToggleHidden()
	self:SetHidden(_, not QuestItemBar.db.profile.bar.hide)
end

function QuestItemBar:SetUsable(_, value)
	self.db.profile.bar.onlyUsable = value
	--self:UpdateBar()
	self:SendMessage("QUEST_ITEM_BAR_ITEM_UPDATE")
end

function QuestItemBar:OpenConfig()
	if InCombatLockdown() then return end
	InterfaceOptionsFrame_OpenToCategory(self.optionsFrame.Profiles)
	InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
end

function QuestItemBar:SetupOptions()
	self.options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	self.options.args.profiles.order = 2
		
	LibStub("AceConfig-3.0"):RegisterOptionsTable(self.name, QuestItemBar.consoleOptions, {L["QuestItemBar"], L["qib"]})
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("QuestItemBar Profile", self.options.args.profiles)
	
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions(self.name, "QuestItemBar")
	self.optionsFrame.Profiles = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("QuestItemBar Profile", "Profile", "QuestItemBar")
end
